@@ -10,7 +10,7 @@ BEGIN {
 
 BEGIN {
 	$Type::Params::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Params::VERSION   = '1.001_006';
+	$Type::Params::VERSION   = '1.001_007';
 }
 
 use B qw();
@@ -68,8 +68,7 @@ sub _mkslurpy
 sub compile
 {
 	my (@code, %env);
-	@code = 'my (@R, %tmp, $tmp);';
-	push @code, '#placeholder';   # $code[1]
+	push @code, '#placeholder', '#placeholder';  # @code[0,1]
 	
 	my %options    = (ref($_[0]) eq "HASH" && !$_[0]{slurpy}) ? %{+shift} : ();
 	my $arg        = -1;
@@ -78,6 +77,23 @@ sub compile
 	my $max_args   = 0;
 	my $saw_opt    = 0;
 	
+	my $return_default_list = !!1;
+	$code[0] = 'my (%tmp, $tmp);';
+	PARAM: for my $param (@_) {
+		if (HashRef->check($param)) {
+			$code[0] = 'my (@R, %tmp, $tmp);';
+			$return_default_list = !!0;
+			last PARAM;
+		}
+		elsif (not Bool->check($param)) {
+			if ($param->has_coercion) {
+				$code[0] = 'my (@R, %tmp, $tmp);';
+				$return_default_list = !!0;
+				last PARAM;
+			}
+		}
+	}
+		
 	while (@_)
 	{
 		++$arg;
@@ -94,7 +110,10 @@ sub compile
 		
 		if (HashRef->check($constraint))
 		{
-			$constraint = to_TypeTiny($constraint->{slurpy});
+			$constraint = to_TypeTiny(
+				$constraint->{slurpy}
+					or Error::TypeTiny::croak("Slurpy parameter malformed")
+			);
 			push @code,
 				$constraint->is_a_type_of(Dict)     ? _mkslurpy('$_', '%', $constraint => $arg) :
 				$constraint->is_a_type_of(Map)      ? _mkslurpy('$_', '%', $constraint => $arg) :
@@ -115,7 +134,11 @@ sub compile
 			
 			if ($is_optional)
 			{
-				push @code, sprintf 'return @R if $#_ < %d;', $arg;
+				push @code, sprintf(
+					'return %s if $#_ < %d;',
+					$return_default_list ? '@_' : '@R',
+					$arg,
+				);
 				$saw_opt++;
 				$max_args++;
 			}
@@ -178,7 +201,9 @@ sub compile
 			);
 		}
 		
-		push @code, sprintf 'push @R, %s;', $varname;
+		unless ($return_default_list) {
+			push @code, sprintf 'push @R, %s;', $varname;
+		}
 	}
 	
 	if ($min_args == $max_args and not $saw_slurpy)
@@ -209,7 +234,12 @@ sub compile
 		);
 	}
 	
-	push @code, '@R;';
+	if ($return_default_list) {
+		push @code, '@_;';
+	}
+	else {
+		push @code, '@R;';
+	}
 	
 	my $source  = "sub { no warnings; ".join("\n", @code)." };";
 	
@@ -403,12 +433,17 @@ sub multisig
 	
 	for my $i (0 .. $#multi)
 	{
-		my $sig = $multi[$i];
+		my $flag = sprintf('${^TYPE_PARAMS_MULTISIG} = %d', $i);
+		my $sig  = $multi[$i];
 		my @cond;
 		push @cond, sprintf('@_ >= %s', $sig->{min_args}) if defined $sig->{min_args};
 		push @cond, sprintf('@_ <= %s', $sig->{max_args}) if defined $sig->{max_args};
+		if (defined $sig->{max_args} and defined $sig->{min_args}) {
+			@cond = sprintf('@_ == %s', $sig->{min_args})
+				if $sig->{max_args} == $sig->{min_args};
+		}
 		push @code, sprintf('if (%s){', join(' and ', @cond)) if @cond;
-		push @code, sprintf('eval { $r = [ $multi[%d]{closure}->(@_) ] };', $i);
+		push @code, sprintf('eval { $r = [ $multi[%d]{closure}->(@_) ]; %s };', $i, $flag);
 		push @code, 'return(@$r) if $r;';
 		push @code, '}' if @cond;
 	}
@@ -587,8 +622,9 @@ Of course, some people like to use C<shift> for the invocant:
 
    sub dump
    {
-      state $check = compile( Int );
       my $self = shift;
+      
+      state $check = compile( Int );
       my ($limit) = $check->(@_);
       
       local $Data::Dumper::Maxdepth = $limit;
@@ -633,11 +669,13 @@ will die if an odd number of elements are slurped in.
 A check may only have one slurpy parameter, and it must be the last
 parameter.
 
+Having a slurpy parameter will slightly slow down your checks.
+
 =head2 Named Parameters
 
 You can use C<compile_named> to accept a hash of named parameters
 
-   use Type::Params qw(compile);
+   use Type::Params qw(compile_named);
    use Types::Standard qw( slurpy Dict Ref Optional Int );
    
    sub dump
@@ -660,7 +698,7 @@ You can use C<compile_named> to accept a hash of named parameters
 Prior to Type::Tiny 1.002000, the recommendation was to use a slurpy
 C<Dict>. This still works, though the error messages you get might not
 be quite so nice, and you don't get the automatic detection of hash
-versus hashref in the input C<< @_ >>.
+versus hashref in the input C<< @_ >>. Oh, and it's usually slower.
 
    use Type::Params qw(compile);
    use Types::Standard qw( slurpy Dict Ref Optional Int );
@@ -699,6 +737,8 @@ For this, you can still use the C<< slurpy Dict >> hack...
          ],
       );
       my ($string, $arg) = $check->(@_);
+      
+      ...;
    }
    
    my_print("Hello World", colour => "blue");
@@ -756,6 +796,8 @@ Note that the coercion is specified as a string of Perl code. This is usually
 the fastest way to do it, but a coderef is also accepted. Either way, the
 value to be coerced is C<< $_ >>.
 
+Having any coercions will slightly slow down your checks.
+
 =head2 Alternatives
 
 Type::Params can export a C<multisig> function that compiles multiple
@@ -775,8 +817,8 @@ alternative signatures into one, and uses the first one that works:
 
 Coercions, slurpy parameters, etc still work.
 
-There's currently no indication of which of the multiple signatures
-succeeded.
+The magic global C<< ${^TYPE_PARAMS_MULTISIG} >> is set to the index of
+the first signature which succeeded.
 
 The present implementation involves compiling each signature independently,
 and trying them each (in their given order!) in an C<eval> block. The only
@@ -809,18 +851,46 @@ a full example:
             return ($meth, $obj);
          },
       );
+      
       my ($needle, $haystack) = $check->(@_);
       
-      is_HashRef($haystack)  ? $haystack->{$needle} :
-      is_ArrayRef($haystack) ? $haystack->[$needle] :
-      is_Object($haystack)   ? $haystack->$needle   :
-      die;
+      for (${^TYPE_PARAMS_MULTISIG) {
+         return $haystack->[$needle] if $_ == 0;
+         return $haystack->{$needle} if $_ == 1;
+         return $haystack->$needle   if $_ == 2;
+      }
    }
    
    get_from(0, \@array);      # returns $array[0]
    get_from('foo', \%hash);   # returns $hash{foo}
    get_from('foo', $obj);     # returns $obj->foo
 
+=head2 Defaults
+
+Type::Params does not currently offer a built-in way to set defaults
+for a parameter. Setting defaults manually is not especially difficult.
+
+   sub print_coloured {
+      state $check = compile( Str, Optional[Str] );
+      
+      my ($text, $colour) = $check->(@_);
+      $colour //= "black";
+      
+      ...;
+   }
+
+I occasionally get requests for this to work:
+
+   sub print_coloured {
+      state $check = compile( Str, Default[Str, "black"] );
+      
+      my ($text, $colour) = $check->(@_);
+      
+      ...;
+   }
+
+But honestly, I don't find that any clearer.
+
 =head1 COMPARISON WITH PARAMS::VALIDATE
 
 L<Type::Params> is not really a drop-in replacement for L<Params::Validate>;
@@ -872,6 +942,47 @@ any other Type::Tiny type constraints.
 
 =back
 
+=head1 COMPARISON WITH PARAMS::VALIDATIONCOMPILER
+
+L<Params::ValidationCompiler> does basically the same thing as
+L<Type::Params>.
+
+=over
+
+=item *
+
+Params::ValidationCompiler and Type::Params are likely to perform fairly
+similarly. In most cases, recent versions of Type::Params seem to be
+I<slightly> faster, but except in very trivial cases, you're unlikely to
+notice the speed difference. Speed probably shouldn't be a factor when
+choosing between them.
+
+=item *
+
+Type::Params's syntax is more compact:
+
+   state $check = compile(Object, Optional[Int], slurpy ArrayRef);
+
+Versus:
+
+   state $check = validation_for(
+      params => [
+         { type => Object },
+         { type => Int,      optional => 1 },
+         { type => ArrayRef, slurpy => 1 },
+      ],
+   );
+
+=item *
+
+L<Params::ValidationCompiler> offers defaults.
+
+=item *
+
+L<Params::ValidationCompiler> probably has slightly better exceptions.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs to
