@@ -12,7 +12,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Standard::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::VERSION   = '1.001_006';
+	$Types::Standard::VERSION   = '1.001_007';
 }
 
 use Type::Library -base;
@@ -694,10 +694,11 @@ $meta->add_type({
 		require B;
 		my ($regexp, $checker) = @_;
 		my $regexp_string = "$regexp";
-		$_StrMatch{$regexp_string} = $regexp;
 		if ($checker)
 		{
 			return unless $checker->can_be_inlined;
+			
+			$_StrMatch{$regexp_string} = $regexp;
 			return sub
 			{
 				my $v = $_[1];
@@ -710,6 +711,17 @@ $meta->add_type({
 		}
 		else
 		{
+			if ($regexp_string =~ /\A\(\?\^u?:(\.+)\)\z/) {
+				my $length = length $1;
+				return sub { "!ref($_) and length($_)>=$length" };
+			}
+			
+			if ($regexp_string =~ /\A\(\?\^u?:\\A(\.+)\\z\)\z/) {
+				my $length = length $1;
+				return sub { "!ref($_) and length($_)==$length" };
+			}
+			
+			$_StrMatch{$regexp_string} = $regexp;
 			return sub
 			{
 				my $v = $_[1];
@@ -1151,7 +1163,13 @@ hashref and validated:
 
 In either C<Tuple> or C<Dict>, C<< slurpy Any >> can be used to indicate
 that additional values are acceptable, but should not be constrained in
-any way. (C<< slurpy Any >> is an optimized code path.)
+any way. 
+
+C<< slurpy Any >> is an optimized code path. Although the following are
+essentially equivalent checks, the former should run a lot faster:
+
+   Tuple[Int, slurpy Any]
+   Tuple[Int, slurpy ArrayRef]
 
 =begin trustme
 
@@ -1263,6 +1281,10 @@ You can optionally provide a type constraint for the array of subexpressions:
          ],
       ];
 
+On certain versions of Perl, type constraints of the forms
+C<< StrMatch[qr/../ >> and C<< StrMatch[qr/\A..\z/ >> with any number
+of intervening dots can be optimized to simple length checks.
+
 =item C<< Enum[`a] >>
 
 As per MooX::Types::MooseLike::Base:
